import pandas as pd
from optichill import bas_filter


def test_train_single_plt():
    dat_folder = 'data'
    train_filenames = ['Plt1 h 2017-05.csv']
    test_filenames = ['Plt1 h 2017-11.csv']
    keys = 'data/Plt1 Points List.xlsx'

    df_train, df_test = bas_filter.train_single_plt(
        dat_folder, train_filenames, test_filenames,
        keys, include_alarms=False
    )

    assert len(df_train.columns.tolist()) == len(df_test.columns.tolist()), (
        "the training and testing sets have different numbers of descriptors!"
    )

    return


def test_import_and_filter():

    dat_folder = 'data'
    string = ['Plt1 h 2017-05.csv', 'Plt1 h 2017-11.csv']
    keys = 'data/Plt1 Points List.xlsx'

    df = bas_filter.import_and_filter(
        dat_folder, string, keys, include_alarms=False
    )

    assert df.shape[0] != 0, (
        "you've filtered out all of your data!"
    )
    assert df.shape[1] != 0, (
        "you have no descriptors, check your filtering!"
    )

    return


def test_data_import():
    dat_folder = 'data'
    string = 'Plt1 h 2017-05.csv'
    keys = 'data/Plt1 Points List.xlsx'

    df, key = bas_filter.data_import(dat_folder, string, keys)

    assert isinstance(df, pd.DataFrame), (
        "variable containing data is not a pandas dataframe!"
    )
    assert isinstance(key, pd.DataFrame), (
        "variable containing data is not a pandas dataframe!"
    )

    assert 'DataPointName' in key.columns.tolist(), (
        "The datapoint names are not in the points list!"
    )
    assert 'PointType' in key.columns.tolist(), (
        "the point types are not in the points list!"
    )
    assert 'Units' in key.columns.tolist(), (
        "the units are not in the points list!"
    )

    return


def test_data_BAS():

    dat_folder = 'data'
    string = 'Plt1 h 2017-05.csv'
    keys = 'data/Plt1 Points List.xlsx'
    dim_remove = ['HX1CDWRT', 'HX1CHWST']

    df, key = bas_filter.data_import(dat_folder, string, keys)
    bas = bas_filter.data_BAS(df, key, dim_remove=dim_remove)

    assert isinstance(bas, pd.DataFrame), (
        "variable containing filtered data is not a pandas dataframe!"
    )

    cat_test = bas.columns.tolist()
    # cat_test.remove('timestamp')

    for cat in cat_test:
        assert cat in key['DataPointName'].T.tolist(), (
            cat + " is not in the points list!"
        )

    return


def test_alarm_filter():
    dat_folder = 'data'
    string = 'Plt1 h 2017-05.csv'
    keys = 'data/Plt1 Points List.xlsx'
    dim_remove = []

    df, key = bas_filter.data_import(dat_folder, string, keys)
    bas = bas_filter.data_BAS(df, key, dim_remove=dim_remove)
    bas_alm = bas_filter.alarm_filter(bas, key)

    assert "Normal/Alarm" not in bas_alm.columns.tolist(), (
        "there are still alarms in the dataset!"
    )

    return



from optichill import bas_filter
from optichill import GBM_model
import os


def test_train_model():
    """
    This functions tests the train_model function.
    """
    dirname = os.path.dirname(__file__)
    dat_folder = os.path.join(dirname, '../../data')
    list_train = ['Plt1 h 2017-05.csv']
    list_test = ['Plt1 h 2017-11.csv']
    keys = 'data/Plt1 Points List.xlsx'
    df_train, df_test = bas_filter.train_single_plt(
        dat_folder, list_train, list_test,
        keys, include_alarms=False
    )
    ytrain = df_train['kW/Ton']
    ytest = df_test['kW/Ton']
    xtrain = df_train.drop(['kW/Ton'], axis=1)
    xtest = df_test.drop(['kW/Ton'], axis=1)
    test_score = GBM_model.train_model(xtrain, ytrain, xtest, ytest)
    # Assertion to ensure the test score is being returned as a number
    assert isinstance(test_score, float), (
        'Calculation of the test score is incorrect.'
    )
    try:
        GBM_model.train_model(
            xtrain, ytrain, xtest, ytest, feat_filename=2
        )
    except(AssertionError):
        pass
    else:
        raise Exception('Assertion Error not raised. Fix it.')
    return


def test_predict_model():
    """
    This function tests the predict_model function.
    """
    return


def test_feature_importance_list():
    """
    This function tests the feature_importance_list function.
    """
    # Assertion that the string is a csv file
    dirname = os.path.dirname(__file__)
    dat_folder = os.path.join(dirname, '../../data')
    list_train = ['Plt1 h 2017-05.csv']
    list_test = ['Plt1 h 2017-11.csv']
    keys = 'data/Plt1 Points List.xlsx'
    df_train, df_test = bas_filter.train_single_plt(
        dat_folder, list_train, list_test,
        keys, include_alarms=False
    )
    ytrain = df_train['kW/Ton']
    ytest = df_test['kW/Ton']
    xtrain = df_train.drop(['kW/Ton'], axis=1)
    xtest = df_test.drop(['kW/Ton'], axis=1)
    GBM_model.train_model(xtrain, ytrain, xtest, ytest)
    try:
        GBM_model.feature_importance_list(
            'files.txt', xtest
        )
    except(Exception):
        pass
    else:
        raise Exception('.csv Exception not raised. Fix it.')
    return



import pandas as pd
import numpy as np
from sklearn.ensemble import GradientBoostingRegressor
from sklearn.metrics import r2_score
import pickle


def train_model(
    xtrain_df, ytrain_df, xtest_df, ytest_df, num_params=500, max_dp=6,
    min_samp_splt=2, lrn_rt=0.01, losss='ls', feat_filename=None
):
    """
    This function takes dataframes of the x and y values for training and
    testing and uses the scikit learn toolbox to train a Gradient Boosting
    Regressor model with preset parameters that can be specified. It returns
    a trained model and the r2 value of the trained model. There is an option
    to create a normalized list of the features to a user-named Excel file.
    feat_filename should be a csv file string.
    """
    # Reshaping the inputs into numpy arrays for the scikit learn module.
    xtrain = xtrain_df.values
    ytrain = ytrain_df.values
    xtest = xtest_df.values
    ytest = ytest_df.values
    # Defining the parameters of GBM
    params = {
        'n_estimators': num_params, 'max_depth': max_dp, 'min_samples_split':
        min_samp_splt, 'learning_rate': lrn_rt, 'loss': losss
        }
    model = GradientBoostingRegressor(**params)
    model.fit(xtrain, ytrain)
    ypred = model.predict(xtest)
    test_score = r2_score(ytest, ypred)
    filename = 'trained_model.sav'
    pickle.dump(model, open(filename, 'wb'))
    # Feature importance list function call
    if feat_filename:
        assert isinstance(feat_filename, str)

        feature_importance_list(feat_filename, xtest_df)
        return test_score
    else:
        return test_score


def predict_model():
    """
    This function unpickles the saved model and returns it as an object.
    """
    filename = 'trained_model.sav'
    new_model = pickle.load(open(filename, 'rb'))
    return new_model


def feature_importance_list(feat_filename, xtest_df):
    """
    This function takes the trained model and returns an ordered list of the
    relative importance of the features. This will be written to an Excel
    file that the user will specify.
    """
    if '.csv' in feat_filename:
        pass
    else:
        raise Exception(
            'Error: Feature importance file must be a .csv file.'
        )
    filename = 'trained_model.sav'
    new_model = pickle.load(open(filename, 'rb'))
    feature_importance = new_model.feature_importances_
    feature_importance = 100.0 * (
        feature_importance / feature_importance.max()
    )
    sorted_idx = np.argsort(feature_importance)
    # pos = np.arange(sorted_idx.shape[0]) + 0.5
    x = xtest_df.columns[sorted_idx]
    imp = feature_importance[sorted_idx]
    df_feats = pd.DataFrame({'Feature Name': x, 'Feature Importance': imp})
    df_feats.to_csv(feat_filename)
    return print(
        "The feature importance list was created as", feat_filename
    )



"""A setuptools-based setup module adapted from the Python Packaging
Authority's sample project.

See:
https://packaging.python.org/en/latest/distributing.html
https://github.com/pypa/sampleproject
"""

# Always prefer setuptools over distutils
from setuptools import setup, find_packages
# To use a consistent encoding
from codecs import open
from os import path

here = path.abspath(path.dirname(__file__))

# Get the long description from the README file
with open(path.join(here, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

# Arguments marked as "Required" below must be included for upload to PyPI.
# Fields marked as "Optional" may be commented out.

setup(
    # Install this project using:
    # $ pip install optichill
    #
    # It lives on PyPI: https://pypi.org/project/optichill/
    name='optichill',

    # Versions should comply with PEP 440:
    # https://www.python.org/dev/peps/pep-0440/
    version='1.0.1',  # Required

    # This is a one-line description or tagline of what your project does. This
    # corresponds to the "Summary" metadata field:
    # https://packaging.python.org/specifications/core-metadata/#summary
    description='Machine Learning Package for chiller plant efficiency',
    # Required
    url='https://github.com/optichill/optichill',
    author='Optimum energy DIRECT team',
    author_email='tacohen@uw.edu',
    packages=find_packages(),


    install_requires=[
        'numpy',
        'pandas',
        'scipy',
        'scikit-learn',
        'matplotlib',
        'seaborn',
        'sklearn'
    ]

)



from __future__ import absolute_import, division, print_function

__all__ = ["GBM_model", "bas_filter"]

# from .version import __version__
from optichill import bas_filter
# from optichill import GBM_model

# __all__ = [__version__, index_return]



import pandas as pd
import glob
import os

#  Overall Function  #


def train_single_plt(
    folder, train_filenames, test_filenames,
    keys, include_alarms=True, dim_remove=[], time_list=[]
):
    """imports test and train plant data and creates data frames
    with filtered data
    Input:
    folder = path to location of raw data files
    train_filenames = list of filenames to be used as the training set.
    test_filenames = list of filenames to be used as the testing set.
    keys = file name from current directory containing the keys spreadsheet
    include_alarms = include or remove alarms from dataset (default = True)
    dim_remove = list of descriptors to remove from dataset (default = NULL)

    Output:
    df_bas1_train = dataframe containing filtered training data
    df_bas1_test = dataframe containing filtered testS data"""

    print('Filtering Training Set')

    bas1_train = import_and_filter(
        folder, train_filenames, keys, include_alarms=include_alarms,
        dim_remove=dim_remove, time_list=time_list
    )

    print('Filtering Test Set')

    bas1_test = import_and_filter(
        folder, test_filenames, keys, include_alarms=include_alarms,
        dim_remove=dim_remove, time_list=time_list
    )

    # matches training and testing columns
    vals_test = [x for x in bas1_test.columns if x in bas1_train.columns]
    df_bas1_test = bas1_test[vals_test]
    df_bas1_train = bas1_train[vals_test]

    return df_bas1_train, df_bas1_test

#  Sub Components  #


def import_and_filter(
    folder, train_filenames, keys, include_alarms=True, dim_remove=[],
    time_list=[]
):
    """imports plant data and creates data frames with filtered data and keys

    Input:
    folder = folder containing raw data.
    train_filenames = list of filenames to be used as dataset.
    keys = file name from current directory containing the keys spreadsheet
    include_alarms = include or remove alarms from dataset (default = True)
    dim_remove = list of descriptors to remove from dataset (default = NULL)

    Output:
    bas1 = dataframe containing filtered plant data"""

    # asserts that the list of filenames is indeed a list
    assert isinstance(train_filenames, list), (
        "train_filenames needs to be a list!"
    )

    # creates blank dataframe for appending
    df = pd.DataFrame()

    # imports and concats all datasets
    for train_string in train_filenames:

        dfloop, key = data_import(
            folder, train_string, keys
        )
        df = pd.concat([df, dfloop], ignore_index=True)

    # removes optional timeframes

    # df_time = time_filter(df, time_list)

    # removes categories of descrdiptors from the dataset
    bas = data_BAS(df, key, dim_remove=dim_remove)

    # filters out alarms
    if include_alarms is False:
        bas1 = alarm_filter(bas, key)
    else:
        bas1 = bas

    # verification statement
    print(
        'Filtered data contains ' + str(bas1.shape[0]) + ' points and '
        + str(bas1.shape[1]) + ' dimensions.'
    )

    return bas1


def data_import(dat_folder, string, keys):
    """imports plant data and creates data frames with raw data and keys

    dat_folder = folder containing raw data.
    string = prefix of the csv files to be imported.
    keys = file name from current directory containing the keys spreadsheet

    Output:
    df = dataframe containing plant data
    key = dataframe containing descriptor key"""

    # Assert that string is .csv

    assert string.endswith('.csv'), (
        "file name " + string + " is not a csv!"
    )

    # assert that the path inputted exists
    assert os.path.exists(dat_folder), (
        "path " + dat_folder + " does not exist!"
    )

    # extracts file names
    dat_list = glob.glob(os.path.join(dat_folder, string))
    print(dat_list)

    # reads and appends content from file to a data frame
    df = pd.read_csv(dat_list[0])

    key = pd.read_excel(keys)

    return df, key


def data_BAS(df, key, dim_remove=[]):
    '''Filters out descriptors containing NaN values, calculated descriptors,
     and miscelaneous descriptors specified by the user.

    Input:
    df = dataframe containing plant data
    key = dataframe containing descriptor key
    dim_remove = list of descriptors to remove from dataset (default = NULL)

    Output:
    bas = dataframe filtered for descriptors and NaN values'''

    # finds keys from categories BAS, Chiller, Condenser Water Pump
    # and Cooling Tower Cell
    keys = []

    key_list = ["BAS", "Chiller", "Condenser Water Pump", "Cooling Tower Cell"]

    # assert that the key list entries are in the PointType
    for key_test in key_list:
        assert key_test in key['PointType'].T.tolist(), (
            "the PointType " + key_test +
            " Could not be found in the points list."
        )

    for k in range(0, len(key_list)):
        key_loop = key.loc[
            key['PointType'].str.contains(key_list[k]),
            'DataPointName'
        ].T.tolist()
        keys += key_loop

    # assert that the key list is in fact a list
    assert isinstance(keys, list), (
        'The PointType list is not a list.'
    )

    # removes DataPointNames that containt the prefix CHWV and kW
    kw = [x for x in keys if 'kW' not in x]
    vals = [x for x in kw if not x.startswith('CHWV')]

    # optional dimension remover
    for dim in dim_remove:

        # asserts that the dimension is in the descriptors list
        assert dim in vals, (
            dim + " is not in the descriptors list"
        )
        vals.remove(dim)

    print('Descriptors in the points list that are not in the datasets.')
    for x in vals:
        if x not in df.columns:
            # prints and removes any string not found in the data
            print(x)
            vals.remove(x)

    # headings of the dataset

    vals_new = [x for x in vals if x in df.columns]

    # expresses data using columns specified by the vals list
    bas = df[vals_new+['OptimumControl', 'kW/Ton']]

    print(
        'Original data contains ' + str(df.shape[0]) + ' points and '
        + str(df.shape[1]) + ' dimensions.'
    )

    bas = bas[bas['OptimumControl'] == 1]

    return bas.dropna()


def alarm_filter(bas, key):
    """removes any datapoints with alarms going off or without optimum control

    bas = dataframe containing plant data
    key = dataframe containing descriptor key"""

    # filters kes to select those with alarm units that are also BAS
    key = key.dropna()

    key_alarm = key.loc[
        key['Units'].str.contains("Normal/Alarm"), 'DataPointName'
    ]

    vals = [x for x in key_alarm if x in bas.columns]

    # tests whether an alarm is going off
    bas_start = bas.shape[0]
    for alm in vals:

        # returns dataframe that have no alarms going off
        bas = bas[bas[alm] == 0]

        bas_end = bas.shape[0]

        # compares shape before and after alarm filtering
        if bas_end != bas_start:
            print(
                'A ' + alm + ' was noted and ' + str(bas_start-bas_end)
                + ' datapoints were removed from the dataset.'
            )

    return bas

# two plant test train function
# THIS FUNCTION DOES NOT NEED UNIT TESTS!!!!
# IT IS NOT BEING USED RIGHT NOW!!!


def train_plt_ref(
    train_folder, train_string, train_keys, test_folder,
    test_string, test_keys, include_alarms=True
):
    """imports test and train plant data and creates data frames with
     filtered data

    Input:
    train_folder = folder containing raw data.
    train_string = prefix of the csv files to be imported.
    train_keys = file name from current directory containing the keys
     spreadsheet
    test_folder = folder containing raw data.
    test_string = prefix of the csv files to be imported.
    test_keys = file name from current directory containing keys spreadsheet
    include_alarms = include or remove alarms from dataset (default = True)

    Output:
    df_bas1_train = dataframe containing filtered training plant data
    df_ce bas1_test = dataframe containing filtered test plant data"""
    print('Filtering Training Set')
    df, key = data_import(train_folder, train_string, train_keys)
    bas = data_BAS(df, key)
    if include_alarms is False:
        bas1_train = alarm_filter(bas, key)
    else:
        bas1_train = bas

    print('Filtering Test Set')
    df, key = data_import(test_folder, test_string, test_keys)
    bas = data_BAS(df, key)
    if include_alarms is False:
        bas1 = alarm_filter(bas, key)
    else:
        bas1 = bas

    vals_test = [x for x in bas1.columns if x in bas1_train.columns]
    df_bas1_test = bas1[vals_test]
    df_bas1_train = bas1_train[vals_test]

    return df_bas1_train, df_bas1_test
